from collections import Counter
from unittest.mock import patch
import itertools
import time
from unittest import TestCase, skip
from chibi_dl.site.crunchyroll import Crunchyroll, Serie, Episode
from chibi_dl.site.crunchyroll.subtitles import Subtitle
from chibi_dl.site.crunchyroll.exceptions import Episode_not_have_media
from vcr_unittest import VCRTestCase
from chibi.file.temp import Chibi_temp_path
from chibi.atlas import Chibi_atlas


class Test_crunchyroll( TestCase ):
    def __init__( self, *args, **kwargs ):
        super().__init__( *args, **kwargs )
        self.helper = None
        if self.__class__ != Test_crunchyroll:
            self.run = TestCase.run.__get__( self, self.__class__ )
        else:
            self.run = lambda self, *args, **kwargs: None

    def test_iter_a_serie( self ):
        with self.subTest( "should have urls" ):
            self.assertTrue( self.site.urls )
        series = list( filter(
            lambda x: isinstance( x, Serie ), self.site.urls ) )
        with self.subTest( "at least should have one serie" ):
            self.assertTrue( series )

        serie = series[0]
        episodes = list( itertools.islice( serie, 2 ) )
        with self.subTest( "the series should have episodes" ):
            self.assertTrue( episodes )
            for episode in episodes:
                self.assertIsInstance( episode, Episode )

        subtitles = []
        for episode in episodes:
            with self.subTest( "each episode should have info" ):
                self.assertTrue( episode.info )
                self.assertIsInstance( episode.info, Chibi_atlas )

            with self.subTest( "each episode should have stream" ):
                self.assertTrue( episode.stream )
                self.assertTrue( episode.stream.uri )

            with self.subTest(
                    "the episode should have subtitles", episode=episode ):
                self.assertTrue( episode.subtitles )
                subtitles += episode.subtitles[:2]

            for subtitle in episode.subtitles:
                with self.subTest(
                        "subtitle should have a iso lang", subtitle=subtitle ):
                    iso_lang = subtitle.lang_ISO_639_2
                    self.assertTrue( iso_lang )

        subtitles = subtitles[:2]
        with self.subTest( "the subtitles should have info" ):
            for subtitle in subtitles:
                self.assertTrue( subtitle.info )
                self.assertIsInstance( subtitle.info, Chibi_atlas )

        with self.subTest( "the subtitles should have be tranformed ot ass" ):
            for subtitle in subtitles:
                ass = subtitle.ass
                self.assertIsInstance( ass, str )
                self.assertTrue( ass )

        with self.subTest( "subtitle download should write the ass file" ):
            for subtitle in subtitles:
                download_path = Chibi_temp_path()
                subtitle_path = subtitle.download( download_path )
                self.assertTrue( subtitle_path.exists )
                self.assertEqual( subtitle_path.open().read(), subtitle.ass )

        with self.subTest( "episode should have a stream" ):
            for episode in episodes:
                stream = episode.stream
                self.assertIsNotNone( stream )
                self.assertTrue( stream.uri )

        with self.subTest( "when download the stream should run ffmpeg" ):
            for episode in episodes:
                with patch( 'ffmpeg.run' ) as run:
                    download_path = Chibi_temp_path()
                    stream_path = episode.download_stream( download_path )
                    self.assertIsNotNone( stream_path )
                    run.assert_called()

        for episode in episodes:
            with self.subTest(
                    "download episode without subs", episode=episode ):
                with patch( 'ffmpeg.run' ) as run:
                    download_path = Chibi_temp_path()
                    stream_path = episode.download(
                        download_path, download_subtitles=False )
                    self.assertFalse( list( download_path.ls() ) )

        for episode in episodes:
            with self.subTest(
                    "download episode with subs", episode=episode ):
                with patch( 'ffmpeg.run' ) as run:
                    download_path = Chibi_temp_path()
                    stream_path = episode.download(
                        download_path, download_subtitles=True )
                    subtitles = list( download_path.ls() )
                    self.assertEqual(
                        len( episode.subtitles ), len( subtitles ) )

        for episode in episodes:
            with self.subTest(
                    "only one subtitle by episode should have default",
                    episode=episode ):
                defaults = Counter( s.default for s in episode.subtitles  )
                self.assertEqual( defaults[ True ], 1 )


    @skip( "slow" )
    def test_should_pack_the_episode_with_the_subtitles( self ):
        folder = Chibi_temp_path()
        result = self.site.series[0].episodes[0].download( folder )
        m4a = next( folder.find( r".*.m4a" ) )
        self.assertTrue( m4a )
        subtitles = list( folder.find( r".*.ass" ) )
        self.assertGreater( len( subtitles ), 1 )
        pack = self.site.series[0].episodes[0].pack( folder )
        mkv = next( folder.find( r".*.mkv" ) )


@skip( "quiero migrar esta mierda a otro paquete" )
class Test_crunchyroll_no_dub( VCRTestCase, Test_crunchyroll ):
    def setUp( self ):
        super().setUp()
        self.site = Crunchyroll()
        self.site.append( 'https://www.crunchyroll.com/es/yuruyuri' )
        #self.site.login()

    def tearDown( self ):
        super().tearDown()


"""
class Test_crunchyroll_with_dub( VCRTestCase, Test_crunchyroll ):
    def setUp( self ):
        super().setUp()
        self.site = Crunchyroll(
            url='http://www.crunchyroll.com/es/miss-kobayashis-dragon-maid',
            user='asdf',
            password='1234',
            quality=240 )
"""



#!/usr/bin/env python3
import json
import sys
import logging
import random
from argparse import ArgumentParser

from chibi.file import Chibi_path
from chibi.config import basic_config, load as load_config

from chibi_dl.site import Site

from chibi_dl.site.nhentai.site import Nhentai
from chibi_dl.site.crunchyroll.site import Crunchyroll
from chibi_dl.site.tmofans import TMO_fans


logger_formarter = '%(levelname)s %(name)s %(asctime)s %(message)s'


parser = ArgumentParser(
    description="descarga mangas", fromfile_prefix_chars='@'
)

parser.add_argument(
    "sites", nargs='+', metavar="site",
    help="urls de las series que se quieren descargar" )

parser.add_argument(
    "--user", '-u', dest="user", default="",
    help="usuario del sitio" )

parser.add_argument(
    "--password", '-p', dest="password", default="",
    help="contrasenna del sitio" )

parser.add_argument(
    "--resoulution", '-r', dest="quality", default=240, type=int,
    help="resolucion a descargar" )

parser.add_argument(
    "--only_print", dest="only_print", action="store_true",
    help="define si silo va a imprimir la lista de links o episodios"
)

parser.add_argument(
    "--only_metadata", dest="only_metadata", action="store_true",
    help="se define si solo se queire recolectar los datos y no descargar"
)

parser.add_argument(
    "--only_links", dest="only_print_links", action="store_true",
    help="si se usa solo imprimira las urls"
)

parser.add_argument(
    "--random", dest="random", action="store_true",
    help="procesa las urls en un orden aleatorio"
)

parser.add_argument(
    "--log_level", dest="log_level", default="INFO",
    help="nivel de log",
)

parser.add_argument(
    "-o", "--output", type=Chibi_path, dest="download_path",
    help="ruta donde se guardara el video o manga" )

parser.add_argument(
    "-config_site", type=Chibi_path, dest="config_site",
    help="python, yaml o json archivo con el usuario y password de cada sitio"
)


def main():
    args = parser.parse_args()
    basic_config( args.log_level )

    if args.config_site:
        load_config( args.config_site )

    tmo_fans = TMO_fans( user=args.user, password=args.password, )
    proccessors = [
        Nhentai(),
        Crunchyroll(
            user=args.user, password=args.password,
            quality=args.quality ),
        tmo_fans,
    ]

    for site in args.sites:
        for proccesor in proccessors:
            if proccesor.i_can_proccess_this( site ):
                if proccesor.append( site ):
                    break


    if args.only_metadata:
        for proccesor in proccessors:
            for batch in proccesor:
                for item in batch:
                    json.dump( item.metadata, sys.stdout )
    if args.only_print:
        if args.only_print_links:
            for serie in tmo_fans.series:
                print( serie.url )
        """
        for proccesor in proccessors:
            logger.info( proccesor )
            for batch in proccesor:
                logger.info( proccesor )
                for item in batch:
                    logger.info( proccesor )
                    json.dump( item.metadata, sys.stdout )
        """
    else:
        for serie in tmo_fans.series:
            serie.download( args.download_path )

        """
        for proccesor in proccessors:
            proccesor.login()
            for download in proccesor:
                download.download( args.download_path )
        """


    """
    site = Site(
        user=args.user, password=args.password,
        quality=args.quality )
    if args.random:
        random.shuffle( args.sites )
    site.append( *args.sites )

    if args.only_print:
        site.print( args.only_print_links )
    else:
        site.download( args.download_path )
    """
